@@ -5,7 +5,7 @@
 ;; Author: Daniel Mendler <mail@daniel-mendler.de>
 ;; Maintainer: Daniel Mendler <mail@daniel-mendler.de>
 ;; Created: 2023
-;; Version: 0.8
+;; Version: 0.9
 ;; Package-Requires: ((emacs "27.1") (compat "29.1.4.0"))
 ;; Homepage: https://github.com/minad/jinx
 ;; Keywords: convenience, wp
@@ -36,9 +36,9 @@
 ;; misspelling can be corrected from a list of dictionary words
 ;; presented as a completion menu.
 
-;; Installing Jinx is straight-forward and configuring takes not much
-;; intervention.  Jinx can safely co-exist with Emacs's built-in
-;; spell-checker.
+;; Installing Jinx is straight-forward and configuring should not need
+;; much intervention.  Jinx can be used completely on its own, but can
+;; also safely co-exist with Emacs's built-in spell-checker Ispell.
 
 ;; Jinx's high performance and low resource usage comes from directly
 ;; calling the widely-used API of the Enchant library (see
@@ -87,17 +87,23 @@
                font-lock-doc-face
                font-lock-string-face)
     (conf-mode font-lock-comment-face
-               font-lock-string-face))
+               font-lock-string-face)
+    ;; yaml-mode and yaml-ts-mode are text-modes,
+    ;; while they should better be conf- or prog-modes.
+    (yaml-mode font-lock-comment-face
+               font-lock-string-face)
+    (yaml-ts-mode font-lock-comment-face
+                  font-lock-string-face))
   "Alist of faces per major mode.
 These faces mark regions which should be included in spell
 checking."
   :type '(alist :key-type symbol :value-type (repeat face)))
 
 (defcustom jinx-camel-modes
-  '(java-mode javascript-mode
-    java-ts-mode javascript-ts-mode
-    ruby-mode ruby-ts-mode
-    rust-mode rust-ts-mode)
+  '(java-mode java-ts-mode js-mode js-ts-mode ruby-mode ruby-ts-mode rust-mode
+    rust-ts-mode haskell-mode kotlin-mode swift-mode csharp-mode csharp-ts-mode
+    objc-mode typescript-ts-mode typescript-mode python-mode python-ts-mode
+    dart-mode go-mode go-ts-mode scala-mode groovy-mode)
   "Modes where camelCase or PascalCase words should be accepted.
 Set to t to enable camelCase everywhere."
   :type '(choice (const t) (repeat symbol)))
@@ -106,20 +112,18 @@ Set to t to enable camelCase everywhere."
   '((markdown-mode
      markdown-code-face markdown-html-attr-name-face
      markdown-html-attr-value-face markdown-html-tag-name-face
-     markdown-inline-code-face markdown-link-face
-     markdown-markup-face markdown-plain-url-face
-     markdown-reference-face markdown-url-face)
+     markdown-inline-code-face markdown-link-face markdown-markup-face
+     markdown-plain-url-face markdown-reference-face markdown-url-face)
     (org-mode
-     org-block org-block-begin-line org-block-end-line
-     org-code org-cite org-cite-key org-date org-footnote
-     org-formula org-latex-and-related org-link org-meta-line
-     org-property-value org-ref-cite-face org-special-keyword
-     org-tag org-todo org-todo-keyword-done
-     org-todo-keyword-habt org-todo-keyword-kill
-     org-todo-keyword-outd org-todo-keyword-todo
-     org-todo-keyword-wait org-verbatim
-     org-modern-tag org-modern-date-active
-     org-modern-date-inactive)
+     org-block org-block-begin-line org-block-end-line org-code org-cite
+     org-cite-key org-date org-document-info-keyword org-footnote org-formula
+     org-latex-and-related org-link org-macro org-meta-line org-property-value
+     org-ref-cite-face org-special-keyword org-tag org-todo
+     org-todo-keyword-done org-todo-keyword-habt org-todo-keyword-kill
+     org-todo-keyword-outd org-todo-keyword-todo org-todo-keyword-wait
+     org-verbatim
+     ;; org-modern
+     org-modern-tag org-modern-date-active org-modern-date-inactive)
     (tex-mode
      tex-math font-latex-math-face font-latex-sedate-face
      font-latex-verbatim-face font-lock-function-name-face
@@ -128,14 +132,14 @@ Set to t to enable camelCase everywhere."
      font-lock-function-name-face font-lock-keyword-face
      font-lock-variable-name-face)
     (rst-mode
-     rst-literal rst-external rst-directive rst-definition
-     rst-reference)
+     rst-literal rst-external rst-directive rst-definition rst-reference)
     (sgml-mode
      font-lock-function-name-face font-lock-variable-name-face)
     (emacs-lisp-mode
      font-lock-constant-face font-lock-warning-face)
     (message-mode
-     message-header-name))
+     message-header-cc message-header-name message-header-newsgroups
+     message-header-other message-header-to message-header-xheader))
   "Alist of faces per major mode.
 These faces mark regions which should be excluded in spell
 checking."
@@ -185,7 +189,7 @@ checking."
   "Face used to highlight current misspelling during correction.")
 
 (defface jinx-save
-  '((t :inherit font-lock-builtin-face))
+  '((t :inherit font-lock-negation-char-face))
   "Face used for the save actions during correction.")
 
 (defface jinx-key
@@ -198,25 +202,37 @@ checking."
 
 ;;;; Keymaps
 
-(defvar-keymap jinx-misspelled-map
+(defvar-keymap jinx-overlay-map
   :doc "Keymap attached to misspelled words."
-  "<mouse-1>" #'jinx-correct)
+  "<mouse-1>" #'jinx-correct
+  "M-n" #'jinx-next
+  "M-p" #'jinx-previous
+  "M-$" #'jinx-correct)
 
-(fset 'jinx-misspelled-map jinx-misspelled-map)
+(fset 'jinx-overlay-map jinx-overlay-map)
 
-(defvar-keymap jinx-mode-map
-  :doc "Keymap used when Jinx is active.")
+(defvar-keymap jinx-repeat-map
+  :doc "Repeat keymap for navigation commands."
+  :repeat (:exit (jinx-correct))
+  "M-n" #'jinx-next
+  "M-p" #'jinx-previous
+  "n" #'jinx-next
+  "p" #'jinx-previous
+  "$" #'jinx-correct)
 
 (defvar-keymap jinx-correct-map
   :doc "Keymap active in the correction minibuffer."
   "SPC" #'self-insert-command
-  "M-n" #'jinx-correct-next
-  "M-p" #'jinx-correct-previous
-  "M-$" #'jinx-correct-previous
+  "M-n" #'jinx-next
+  "M-p" #'jinx-previous
+  "M-$" #'jinx-previous
   "0 <t>" #'jinx-correct-select)
 (dotimes (i 9)
   (define-key jinx-correct-map (vector (+ ?1 i)) #'jinx-correct-select))
 
+(defvar-keymap jinx-mode-map
+  :doc "Keymap used when Jinx is active.")
+
 (easy-menu-define jinx-mode-menu jinx-mode-map
   "Menu used when Jinx is active."
   '("Jinx"
@@ -242,6 +258,28 @@ Predicate may return a position to skip forward.")
 (defvar jinx--timer nil
   "Global timer to check pending regions.")
 
+(defvar jinx--base-syntax-table
+  (let ((st (make-syntax-table)))
+    (modify-syntax-entry ?$ "_" st)
+    (modify-syntax-entry ?% "_" st)
+    (modify-syntax-entry '(#x80 . #x9f) "_" st)        ;; Control characters
+    (modify-syntax-entry '(#x2150 . #x2bff) "_" st)    ;; Number Forms - Misc. Arrows
+    (modify-syntax-entry '(#xfe00 . #xfe0f) "_" st)    ;; Variation Selectors
+    (modify-syntax-entry '(#x1cf00 . #x1d7ff) "_" st)  ;; Znamenny Musical - Math. Alpha.
+    (modify-syntax-entry '(#x1ee00 . #x1fbff) "_" st)  ;; Arabic Math. - Legacy Computing
+    st)
+  "Base syntax table for spell checking.")
+
+(defvar jinx--select-keys
+  "123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ"
+  "Quick select keys used by `jinx-correct'.")
+
+(defvar jinx--save-keys
+  `((?@ . ,#'jinx--save-personal)
+    (?* . ,#'jinx--save-file)
+    (?+ . ,#'jinx--save-session))
+  "Keys for save actions used by `jinx-correct'.")
+
 (defvar-local jinx--exclude-faces nil
   "List of excluded faces.")
 
@@ -263,18 +301,9 @@ Predicate may return a position to skip forward.")
 (defvar-local jinx--session-words nil
   "List of words accepted in this session.")
 
-(defvar jinx--select-keys
-  "123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ"
-  "Quick select keys used by `jinx-correct'.")
-
-(defvar jinx--save-keys
-  `((?@ . ,#'jinx--save-personal)
-    (?* . ,#'jinx--save-file)
-    (?+ . ,#'jinx--save-session))
-  "Keys for save actions used by `jinx-correct'.")
-
 ;;;; Declarations for the bytecode compiler
 
+(defvar repeat-mode)
 (defvar jinx-mode)
 (declare-function jinx--mod-check nil)
 (declare-function jinx--mod-add nil)
@@ -288,13 +317,13 @@ Predicate may return a position to skip forward.")
 
 ;;;; Overlay properties
 
-(put 'jinx 'evaporate             t)
-(put 'jinx 'face                  'jinx-misspelled)
-(put 'jinx 'mouse-face            '(jinx-misspelled jinx-highlight))
-(put 'jinx 'modification-hooks    (list #'jinx--overlay-modified))
-(put 'jinx 'insert-in-front-hooks (list #'jinx--overlay-modified))
-(put 'jinx 'insert-behind-hooks   (list #'jinx--overlay-modified))
-(put 'jinx 'keymap                'jinx-misspelled-map)
+(put 'jinx-overlay 'evaporate             t)
+(put 'jinx-overlay 'face                  'jinx-misspelled)
+(put 'jinx-overlay 'mouse-face            '(jinx-misspelled jinx-highlight))
+(put 'jinx-overlay 'modification-hooks    (list #'jinx--overlay-modified))
+(put 'jinx-overlay 'insert-in-front-hooks (list #'jinx--overlay-modified))
+(put 'jinx-overlay 'insert-behind-hooks   (list #'jinx--overlay-modified))
+(put 'jinx-overlay 'keymap                'jinx-overlay-map)
 
 ;;;; Predicates
 
@@ -329,6 +358,11 @@ Predicate may return a position to skip forward.")
 
 ;;;; Internal functions
 
+(defun jinx--in-base-buffer (&rest app)
+  "Apply APP in `buffer-base-buffer', as required by `jit-lock' functions."
+  (with-current-buffer (or (buffer-base-buffer) (current-buffer))
+    (apply app)))
+
 (defun jinx--overlay-modified (overlay &rest _)
   "Delete modified OVERLAY.
 This function is a modification hook for the overlay."
@@ -361,15 +395,6 @@ FLAG must be t or nil."
             (setq pos (jinx--check-region from to retry))
           (setq pos to))))))
 
-(defun jinx--force-check-region (start end)
-  "Enforce spell-check of region between START and END."
-  ;; FIXME `with-delayed-message' is broken in combination with
-  ;; `inhibit-message'. Report this as a bug.
-  (progn ;; with-delayed-message (1 "Fontifying...")
-    (jit-lock-fontify-now))
-  (progn ;; with-delayed-message (1 "Checking...")
-    (jinx--check-region start end)))
-
 (defun jinx--check-region (start end &optional retry)
   "Check region between START and END.
 Optionally RETRY word at given position.  Return updated END
@@ -399,11 +424,11 @@ position."
                   ;; No quote or apostrophe at start or end
                   (while (and (< word-start word-end)
                               (let ((c (char-after word-start)))
-                                (or (= c 39) (= c 8217))))
+                                (or (= c ?') (= c ?’))))
                     (cl-incf word-start))
                   (while (and (< word-start word-end)
                               (let ((c (char-before word-end)))
-                                (or (= c 39) (= c 8217))))
+                                (or (= c ?') (= c ?’))))
                     (cl-decf word-end))
                   (while (< word-start word-end)
                     (let ((subword-end word-end))
@@ -418,7 +443,7 @@ position."
                         ('nil
                          (if (and retry (<= word-start retry subword-end))
                              (setq retry-start word-start retry-end subword-end retry nil)
-                           (overlay-put (make-overlay word-start subword-end) 'category 'jinx))))
+                           (overlay-put (make-overlay word-start subword-end) 'category 'jinx-overlay))))
                       (setq word-start subword-end)))))
               (remove-list-of-text-properties start end '(jinx--pending))
               (when retry-start
@@ -431,7 +456,7 @@ position."
 If VISIBLE is non-nil, only include visible overlays."
   (let ((pt (point)) before overlays)
     (dolist (ov (overlays-in start end))
-      (when (and (eq (overlay-get ov 'category) 'jinx)
+      (when (and (eq (overlay-get ov 'category) 'jinx-overlay)
                  (not (and visible (invisible-p (overlay-start ov)))))
         (push ov overlays)))
     (setq overlays
@@ -441,10 +466,25 @@ If VISIBLE is non-nil, only include visible overlays."
       (push (pop overlays) before))
     (nconc overlays (nreverse before))))
 
+(cl-defun jinx--force-overlays (start end &key visible check)
+  "Return misspelled word overlays between START and END, enforce checking.
+If VISIBLE is non-nil, only include visible overlays.
+If CHECK is non-nil, always check first."
+  (or (and (not check) (jinx--get-overlays start end visible))
+      (progn
+        (let (set-message-function) ;; bug#63253: Broken `with-delayed-message'
+          (with-delayed-message (1 "Fontifying...")
+            (jinx--in-base-buffer #'jit-lock-fontify-now start end))
+          (with-delayed-message (1 "Checking...")
+            (jinx--check-region start end)))
+        (jinx--get-overlays start end visible))
+      (user-error (if visible "No misspelling in visible text"
+                    "No misspelling in whole buffer"))))
+
 (defun jinx--delete-overlays (start end)
   "Delete overlays between START and END."
   (dolist (ov (overlays-in start end))
-    (when (eq 'jinx (overlay-get ov 'category))
+    (when (eq (overlay-get ov 'category) 'jinx-overlay)
       (delete-overlay ov))))
 
 (defun jinx--cleanup ()
@@ -454,7 +494,7 @@ If VISIBLE is non-nil, only include visible overlays."
       (widen)
       (jinx--delete-overlays (point-min) (point-max))
       (remove-list-of-text-properties (point-min) (point-max) '(jinx--pending))
-      (jit-lock-refontify))))
+      (jinx--in-base-buffer #'jit-lock-refontify))))
 
 (defun jinx--mark-pending (start end)
   "Mark region between START and END as pending."
@@ -471,7 +511,7 @@ If VISIBLE is non-nil, only include visible overlays."
 (defun jinx--get-org-language ()
   "Get language from Org #+language keyword."
   (when (and (not (local-variable-p 'jinx-languages))
-             (derived-mode-p 'org-mode))
+             (derived-mode-p #'org-mode))
     (save-excursion
       (save-match-data
         (goto-char (point-min))
@@ -579,7 +619,7 @@ If VISIBLE is non-nil, only include visible overlays."
     (save-restriction
       (widen)
       (dolist (ov (overlays-in (point-min) (point-max)))
-        (when (eq (overlay-get ov 'category) 'jinx)
+        (when (eq (overlay-get ov 'category) 'jinx-overlay)
           (goto-char (overlay-end ov))
           (when (jinx--word-valid-p (overlay-start ov))
             (delete-overlay ov)))))))
@@ -655,14 +695,15 @@ If VISIBLE is non-nil, only include visible overlays."
                    (annotation-function . jinx--correct-annotation))
       (complete-with-action action word str pred))))
 
-(defun jinx--correct (overlay recenter info)
-  "Correct word at OVERLAY, maybe RECENTER and show prompt INFO."
+(defun jinx--correct (overlay info)
+  "Correct word at OVERLAY, maybe show prompt INFO."
   (let* ((word (buffer-substring-no-properties
                 (overlay-start overlay) (overlay-end overlay)))
          (choice
           (jinx--correct-highlight overlay
             (lambda ()
-              (when recenter (recenter))
+              (when (or (< (point) (window-start)) (> (point) (window-end nil t)))
+                (recenter))
               (minibuffer-with-setup-hook
                   #'jinx--correct-setup
                 (or (completing-read
@@ -671,32 +712,35 @@ If VISIBLE is non-nil, only include visible overlays."
                      nil nil nil t word)
                     word)))))
          (len (length choice)))
-    (if-let ((save (and (> len 0) (assq (aref choice 0) jinx--save-keys))))
-        (progn
-          (funcall (cdr save) 'save (car save) (if (> len 1) (substring choice 1) word))
-          (jinx--recheck-overlays))
-      (when-let (((not (equal choice word)))
-                 (start (overlay-start overlay))
-                 (end (overlay-end overlay)))
-        (undo-boundary)
-        (delete-overlay overlay)
-        (goto-char end)
-        (insert-before-markers choice)
-        (delete-region start end)))))
+    (pcase (and (> len 0) (assq (aref choice 0) jinx--save-keys))
+      (`(,key . ,fun)
+       (funcall fun 'save key (if (> len 1) (substring choice 1) word))
+       (jinx--recheck-overlays))
+      ((guard (not (equal choice word)))
+       (jinx--correct-replace overlay choice)))))
+
+(defun jinx--correct-replace (overlay word)
+  "Replace OVERLAY with WORD."
+  (when-let ((start (overlay-start overlay))
+             (end (overlay-end overlay)))
+    (undo-boundary)
+    (delete-overlay overlay)
+    (goto-char end)
+    (insert-before-markers word)
+    (delete-region start end)))
 
 (defun jinx--load-dicts ()
   "Load dictionaries and setup syntax table."
   (setq jinx--dicts (delq nil (mapcar #'jinx--mod-dict
                                       (split-string jinx-languages)))
-        jinx--syntax-table (make-syntax-table))
+        jinx--syntax-table (make-syntax-table jinx--base-syntax-table))
   (unless jinx--dicts
     (message "Jinx: No dictionaries available for %S" jinx-languages))
   (dolist (dict jinx--dicts)
     (cl-loop for c across (jinx--mod-wordchars dict) do
              (modify-syntax-entry c "w" jinx--syntax-table)))
-  (modify-syntax-entry ?$ "_" jinx--syntax-table)
-  (modify-syntax-entry ?% "_" jinx--syntax-table)
   (modify-syntax-entry ?' "w" jinx--syntax-table)
+  (modify-syntax-entry ?’ "w" jinx--syntax-table)
   (modify-syntax-entry ?. "." jinx--syntax-table))
 
 ;;;; Save functions
@@ -721,7 +765,7 @@ If SAVE is non-nil save, otherwise format candidate given action KEY."
     (cl-loop
      for dict in jinx--dicts for idx from 1
      for at = (make-string idx key)
-     for ann = (format "Personal ‘%s’" (car (jinx--mod-describe dict))) nconc
+     for ann = (format "Personal:%s" (car (jinx--mod-describe dict))) nconc
      (delete-consecutive-dups
       (list (jinx--save-action at word ann)
             (jinx--save-action at (downcase word) ann))))))
@@ -786,37 +830,32 @@ With prefix argument GLOBAL change the languages globally."
 If prefix argument ALL non-nil correct all misspellings."
   (interactive "*P")
   (unless jinx-mode (jinx-mode 1))
-  (cl-letf (((symbol-function #'jinx--timer-handler) #'ignore) ;; Inhibit
-            (old-point (and (not all) (point-marker))))
+  (cl-letf* (((symbol-function #'jinx--timer-handler) #'ignore) ;; Inhibit
+             (repeat-mode nil) ;; No repeating of jinx-next and jinx-previous
+             (old-point (point-marker))
+             (overlays
+              (if all
+                  (jinx--force-overlays (point-min) (point-max) :check t)
+                (jinx--force-overlays (window-start) (window-end) :visible t)))
+             (count (length overlays))
+             (idx 0))
+    (when all
+      (push-mark))
     (unwind-protect
-          (let* ((overlays
-                  (if all
-                      (progn
-                        (jinx--force-check-region (point-min) (point-max))
-                        (or (jinx--get-overlays (point-min) (point-max))
-                            (user-error "No misspellings in whole buffer")))
-                    (or (jinx--get-overlays (window-start) (window-end) 'visible)
-                        (progn
-                          (jinx--force-check-region (window-start) (window-end))
-                          (jinx--get-overlays (window-start) (window-end) 'visible))
-                        (user-error "No misspelling in visible text"))))
-                 (count (length overlays))
-                 (idx 0))
-            (when all
-              (push-mark))
-            (while (when-let ((ov (nth idx overlays)))
-                     (let* ((deleted (not (overlay-buffer ov)))
-                            (skip
-                             (catch 'jinx--correct
-                               (unless deleted
-                                 (jinx--correct
-                                  ov all
-                                  (and all (format " (%d of %d)" (1+ idx) count)))))))
-                       (cond
-                        ((integerp skip) (setq idx (mod (+ idx skip) count)))
-                        ((or all deleted) (cl-incf idx)))))))
-      (when old-point (goto-char old-point))
-      (jit-lock-refontify))))
+        (while (when-let ((ov (nth idx overlays)))
+                 (let* ((deleted (not (overlay-buffer ov)))
+                        (skip
+                         (catch 'jinx--goto
+                           (unless deleted
+                             (jinx--correct
+                              ov (and all (format " (%d of %d)" (1+ idx) count)))))))
+                   (cond
+                    ((integerp skip) (setq idx (mod (+ idx skip) count)))
+                    ((or all deleted) (cl-incf idx))))))
+      (if all
+          (jinx--in-base-buffer #'jit-lock-refontify)
+        (goto-char old-point)
+        (jinx--in-base-buffer #'jit-lock-refontify (window-start) (window-end))))))
 
 (defun jinx-correct-select ()
   "Quick selection key for corrections."
@@ -831,16 +870,23 @@ If prefix argument ALL non-nil correct all misspellings."
     (delete-minibuffer-contents)
     (insert word)
     (exit-minibuffer)))
+(put #'jinx-correct-select 'completion-predicate #'ignore)
 
-(defun jinx-correct-next (n)
-  "Skip to Nth next misspelling."
+(defun jinx-next (n)
+  "Go to to Nth next misspelling."
   (interactive "p")
-  (throw 'jinx--correct n))
+  (unless (= n 0)
+    (if (minibufferp)
+        (throw 'jinx--goto n)
+      (let ((ov (jinx--force-overlays (point-min) (point-max))))
+        (unless (or (> n 0) (<= (overlay-start (car ov)) (point) (overlay-end (car ov))))
+          (cl-incf n))
+        (goto-char (overlay-end (nth (mod n (length ov)) ov)))))))
 
-(defun jinx-correct-previous (n)
-  "Skip to Nth previous misspelling."
+(defun jinx-previous (n)
+  "Go to to Nth previous misspelling."
   (interactive "p")
-  (throw 'jinx--correct (- n)))
+  (jinx-next (- n)))
 
 ;;;###autoload
 (define-minor-mode jinx-mode
@@ -849,6 +895,9 @@ If prefix argument ALL non-nil correct all misspellings."
   :group 'jinx
   :keymap jinx-mode-map
   (cond
+   ((buffer-base-buffer) ;; Do not enable in indirect buffers
+    (when jinx-mode
+      (jinx-mode -1)))
    (jinx-mode
     (jinx--load-module)
     (let ((enable-local-variables :safe))
@@ -861,7 +910,7 @@ If prefix argument ALL non-nil correct all misspellings."
           jinx--include-faces (jinx--mode-list jinx-include-faces)
           jinx--exclude-faces (jinx--mode-list jinx-exclude-faces)
           jinx--camel (or (eq jinx-camel-modes t)
-                          (seq-some #'derived-mode-p jinx-camel-modes))
+                          (apply #'derived-mode-p jinx-camel-modes))
           jinx--session-words (split-string jinx-local-words))
     (jinx--load-dicts)
     (add-hook 'window-state-change-hook #'jinx--reschedule nil t)
@@ -886,8 +935,9 @@ If prefix argument ALL non-nil correct all misspellings."
   "Turn `jinx-mode' on."
   (when (and (not (or noninteractive
                       buffer-read-only
+                      (buffer-base-buffer) ;; Do not enable in indirect buffers
                       (eq (aref (buffer-name) 0) ?\s)))
-             (seq-some #'derived-mode-p jinx-include-modes))
+             (apply #'derived-mode-p jinx-include-modes))
     (jinx-mode 1)))
 
 (provide 'jinx)
